use std::fmt::Display;
use std::str::FromStr;

use anyhow::{anyhow, Result};

use engine::prelude::*;

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct NPCNormal;

impl EntityComponent for NPCNormal {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for NPCNormal {
    fn debug_name() -> &'static str {
        "NPCNormal"
    }
}

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct NPCElite;

impl EntityComponent for NPCElite {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for NPCElite {
    fn debug_name() -> &'static str {
        "NPCElite"
    }
}

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct NPCBoss;

impl EntityComponent for NPCBoss {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for NPCBoss {
    fn debug_name() -> &'static str {
        "NPCBoss"
    }
}

#[derive(Debug, Clone, Copy, Default, PartialEq, Eq)]
pub enum NPCType {
    #[default]
    Normal,
    Elite,
    Boss,
}

impl From<NPCNormal> for NPCType {
    fn from(_: NPCNormal) -> Self {
        Self::Normal
    }
}

impl From<NPCElite> for NPCType {
    fn from(_: NPCElite) -> Self {
        Self::Elite
    }
}

impl From<NPCBoss> for NPCType {
    fn from(_: NPCBoss) -> Self {
        Self::Boss
    }
}

impl FromStr for NPCType {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self> {
        match s.to_lowercase().as_str() {
            "normal" => Ok(Self::Normal),
            "elite" => Ok(Self::Elite),
            "boss" => Ok(Self::Boss),

            _ => Err(anyhow!("could not parse {s} as NPCType")),
        }
    }
}

impl Display for NPCType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match *self {
            NPCType::Normal => write!(f, "normal"),
            NPCType::Elite => write!(f, "elite"),
            NPCType::Boss => write!(f, "boss"),
        }
    }
}
